@@ -36,6 +36,8 @@
 #define POLL_TIMEOUT 300
 #define TRYLOCK_TIMEOUT 50
 
+#define CHECK_EXIT { if (l->terminated != FALSE) { g_mutex_unlock (l->mutex); goto exit_thread; } }
+
 static gpointer		 parent_class = NULL;
 
 typedef struct {
@@ -99,6 +101,7 @@ static void phonemgr_listener_thread (PhonemgrListener *l);
 
 enum {
 	MESSAGE_SIGNAL,
+	REPORT_STATUS_SIGNAL,
 	STATUS_SIGNAL,
 	CALL_STATUS_SIGNAL,
 	BATTERY_SIGNAL,
@@ -174,6 +177,17 @@ phonemgr_listener_class_init (PhonemgrListenerClass *klass)
 			      3,
 			      G_TYPE_STRING, G_TYPE_ULONG, G_TYPE_STRING);
 
+	phonemgr_listener_signals[REPORT_STATUS_SIGNAL] =
+		g_signal_new ("report-status",
+			      G_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (PhonemgrListenerClass, report_status),
+			      NULL, NULL,
+			      phonemgr_marshal_VOID__STRING_ULONG_INT,
+			      G_TYPE_NONE,
+			      3,
+			      G_TYPE_STRING, G_TYPE_ULONG, G_TYPE_INT);
+
 	phonemgr_listener_signals[STATUS_SIGNAL] =
 		g_signal_new ("status",
 			      G_OBJECT_CLASS_TYPE (klass),
@@ -235,6 +249,40 @@ phonemgr_listener_emit_status (PhonemgrListener *l, PhonemgrListenerStatus statu
 		       0, status);
 }
 
+static void
+phonemgr_listener_emit_report_status (PhonemgrListener *l, gn_sms *message)
+{
+	time_t time;
+	char *sender;
+	PhonemgrListenerReportStatus status;
+
+	switch (message->user_data[0].dr_status) {
+	case GN_SMS_DR_Status_Delivered:
+		status = PHONEMGR_LISTENER_REPORT_DELIVERED;
+		break;
+	case GN_SMS_DR_Status_Pending:
+		status = PHONEMGR_LISTENER_REPORT_PENDING;
+		break;
+	case GN_SMS_DR_Status_Failed_Temporary:
+		status = PHONEMGR_LISTENER_REPORT_FAILED_TEMPORARY;
+		break;
+	case GN_SMS_DR_Status_Failed_Permanent:
+		status = PHONEMGR_LISTENER_REPORT_FAILED_PERMANENT;
+		break;
+	default:
+		return;
+	}
+
+	time = gn_timestamp_to_gtime (message->smsc_time);
+	sender = g_strdup (message->remote.number);
+
+	g_signal_emit (G_OBJECT (l),
+		       phonemgr_listener_signals[REPORT_STATUS_SIGNAL],
+		       0, sender, time, status);
+
+	g_free (sender);
+}
+
 static void
 phonemgr_listener_emit_call_status (PhonemgrListener *l,
 				    PhonemgrListenerCallStatus status,
@@ -359,25 +407,43 @@ phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **error)
 	channel = -1;
 
 	if (phonemgr_utils_address_is (device) == PHONEMGR_CONNECTION_BLUETOOTH) {
-		channel = phonemgr_utils_get_channel (device);
+		channel = phonemgr_utils_get_serial_channel (device);
 		if (channel < 0) {
 			//FIXME
 			return FALSE;
 		}
 	}
 
+	phonemgr_utils_free (l->phone_state);
 	l->phone_state = phonemgr_utils_connect (device, NULL, channel, l->debug, error);
 	if (l->phone_state == NULL) {
 		//FIXME
 		return FALSE;
 	}
 
+	g_free (l->driver);
 	l->driver = phonemgr_utils_guess_driver (l->phone_state, device, error);
 	if (l->driver == NULL) {
 		//FIXME
+		phonemgr_utils_disconnect (l->phone_state);
+		phonemgr_utils_free (l->phone_state);
 		return FALSE;
 	}
 
+	/* Do we need to see gnapplet driver? */
+	if (strcmp (l->driver, PHONEMGR_DEFAULT_GNAPPLET_DRIVER) == 0) {
+		channel = phonemgr_utils_get_gnapplet_channel (device);
+		if (channel < 0) {
+			phonemgr_utils_disconnect (l->phone_state);
+			phonemgr_utils_free (l->phone_state);
+			g_free (l->driver);
+			l->driver = NULL;
+			//FIXME
+			g_message ("Couldn't connect to the device, gnapplet needed but not running");
+			return FALSE;
+		}
+	}
+
 	/* Need a different driver? then reconnect */
 	if (strcmp (l->driver, PHONEMGR_DEFAULT_DRIVER) != 0
 	    && strcmp (l->driver, PHONEMGR_DEFAULT_USB_DRIVER) != 0) {
@@ -412,11 +478,18 @@ phonemgr_listener_push (PhonemgrListener *l)
 	if (signal == NULL)
 		return FALSE;
 
-	if (signal->type == MESSAGE_SIGNAL) {
+	switch (signal->type) {
+	case MESSAGE_SIGNAL:
 		g_message ("emitting message");
 		phonemgr_listener_emit_message (l, signal->message);
 		g_free (signal->message);
-	} else if (signal->type == CALL_STATUS_SIGNAL) {
+		break;
+	case REPORT_STATUS_SIGNAL:
+		g_message ("emitting delivery report status");
+		phonemgr_listener_emit_report_status (l, signal->message);
+		g_free (signal->message);
+		break;
+	case CALL_STATUS_SIGNAL:
 		g_message ("emitting call status");
 		phonemgr_listener_emit_call_status (l, signal->call->status,
 						    signal->call->number,
@@ -424,13 +497,15 @@ phonemgr_listener_push (PhonemgrListener *l)
 		g_free (signal->call->number);
 		g_free (signal->call->name);
 		g_free (signal->call);
-	} else if (signal->type == BATTERY_SIGNAL) {
+		break;
+	case BATTERY_SIGNAL:
 		g_message ("emitting battery");
 		phonemgr_listener_emit_battery (l,
 						(int) signal->battery->batterylevel,
 						signal->battery->powersource != GN_PS_BATTERY);
 		g_free (signal->battery);
-	} else {
+		break;
+	default:
 		g_assert_not_reached ();
 	}
 
@@ -444,9 +519,18 @@ phonemgr_listener_new_sms_cb (gn_sms *message, struct gn_statemachine *state, vo
 {
 	PhonemgrListener *l = (PhonemgrListener *) user_data;
 	AsyncSignal *signal;
+	int type;
+
+	/* Check that it's a type of message we support */
+	if (message->type == GN_SMS_MT_DeliveryReport)
+		type = REPORT_STATUS_SIGNAL;
+	else if (message->type == GN_SMS_MT_Deliver)
+		type = MESSAGE_SIGNAL;
+	else
+		return;
 
 	signal = g_new0 (AsyncSignal, 1);
-	signal->type = MESSAGE_SIGNAL;
+	signal->type = type;
 	/* The message is allocated on the stack in the driver, so copy it */
 	signal->message = g_memdup (message, sizeof (gn_sms));
 
@@ -703,16 +787,23 @@ static void
 phonemgr_listener_thread (PhonemgrListener *l)
 {
 	g_mutex_lock (l->mutex);
+	CHECK_EXIT;
 	phonemgr_listener_get_own_details (l);
+	CHECK_EXIT;
 	phonemgr_listener_set_sms_notification (l, TRUE);
+	CHECK_EXIT;
 	phonemgr_listener_set_call_notification (l, TRUE);
+	CHECK_EXIT;
 	g_mutex_unlock (l->mutex);
 
 	while (l->terminated == FALSE) {
 		if (g_mutex_trylock (l->mutex) != FALSE) {
+			CHECK_EXIT;
 			if (l->supports_sms_notif != FALSE)
 				phonemgr_listener_sms_notification_soft_poll (l);
+			CHECK_EXIT;
 			phonemgr_listener_call_notification_poll (l);
+			CHECK_EXIT;
 			phonemgr_listener_battery_poll (l);
 
 			g_mutex_unlock (l->mutex);
@@ -722,6 +813,7 @@ phonemgr_listener_thread (PhonemgrListener *l)
 		}
 	}
 
+exit_thread:
 	g_mutex_lock (l->mutex);
 	phonemgr_listener_set_sms_notification (l, FALSE);
 	phonemgr_listener_set_call_notification (l, FALSE);
@@ -771,7 +863,9 @@ phonemgr_listener_answer_call (PhonemgrListener *l)
 
 void
 phonemgr_listener_queue_message (PhonemgrListener *l,
-		const char *number, const char *message)
+				 const char *number,
+				 const char *message,
+				 gboolean delivery_report)
 {
 	char *mstr, *iso;
 	GError *err = NULL;
@@ -790,9 +884,7 @@ phonemgr_listener_queue_message (PhonemgrListener *l,
 
 	/* Default GSM alphabet is a subset of ISO8859-1,
 	 * so try that */
-	iso = g_convert (message, -1,
-			 "ISO8859-1", "UTF-8",
-			 NULL, NULL, &err);
+	iso = g_convert (message, -1, "ISO8859-1", "UTF-8", NULL, NULL, &err);
 	if (err != NULL) {
 		g_clear_error (&err);
 		g_free (iso);
@@ -835,6 +927,9 @@ phonemgr_listener_queue_message (PhonemgrListener *l,
 	}
 	g_free (l->phone_state->data.message_center);
 
+	/* Set whether to get a delivery report */
+	sms.delivery_report = delivery_report;
+
 	/* Set the message data */
 	sms.user_data[0].type = GN_SMS_DATA_Text;
 	g_strlcpy ((char *) sms.user_data[0].u.text, mstr, strlen (mstr) + 1);
@@ -925,10 +1020,10 @@ phonemgr_listener_poll (PhonemgrListener *l)
 
 		if (g_rand_boolean (rand)) {
 			sender = "+09876 543-21";
-			text = "This is my other t\303\251st, this is NOT a supa test";
+			text = "This is my other t\303\251st, this is NOT a supa test: http://live.gnome.org/PhoneManager";
 		} else {
 			sender = "+01234 567-89";
-			text = "This is my test, this is my supa test with i18n chars: \303\270\317\216\307\252";
+			text = "This is my test, this is my supa test with i18n chars: \303\270\317\216\307\252 (http://live.gnome.org/PhoneManager)";
 		}
 		g_rand_free (rand);
 
@@ -970,9 +1065,12 @@ phonemgr_listener_disconnect (PhonemgrListener *l)
 
 void
 phonemgr_listener_queue_message (PhonemgrListener *listener,
-        const char *number, const char *message)
+				 const char *number,
+				 const char *message,
+				 gboolean delivery_report)
 {
-	g_message ("[DUMMY] sending message to %s: %s", number, message);
+	g_message ("[DUMMY] sending message to %s: %s (Delivery report: %s)",
+		   number, message, delivery_report ? "Yes" : "No");
 }
 
 gboolean
