@@ -42,6 +42,13 @@ typedef enum {
 	PHONEMGR_LISTENER_ERROR
 } PhonemgrListenerStatus;
 
+typedef enum {
+	PHONEMGR_LISTENER_REPORT_DELIVERED,
+	PHONEMGR_LISTENER_REPORT_PENDING,
+	PHONEMGR_LISTENER_REPORT_FAILED_TEMPORARY,
+	PHONEMGR_LISTENER_REPORT_FAILED_PERMANENT
+} PhonemgrListenerReportStatus;
+
 typedef enum {
 	PHONEMGR_LISTENER_CALL_INCOMING,
 	PHONEMGR_LISTENER_CALL_ONGOING,
@@ -59,6 +66,7 @@ struct _PhonemgrListenerClass
 {
   GObjectClass	parent_class;
   void (* message) (PhonemgrListener *l, char *phone, time_t timestamp, char *message);
+  void (* report_status) (PhonemgrListener *l, char *phone, time_t timestamp, PhonemgrListenerReportStatus status);
   void (* status) (PhonemgrListener *l, PhonemgrListenerStatus status);
   void (* call_status) (PhonemgrListener *l, PhonemgrListenerCallStatus status, const char *phone, const char *name);
   void (* battery) (PhonemgrListener *l, int percent, gboolean on_ac);
@@ -73,7 +81,8 @@ gboolean phonemgr_listener_connect	(PhonemgrListener *listener,
 void phonemgr_listener_disconnect	(PhonemgrListener *listener);
 void phonemgr_listener_queue_message	(PhonemgrListener *listener,
 					 const char *number,
-					 const char *message);
+					 const char *message,
+					 gboolean deliver_report);
 void phonemgr_listener_cancel_call	(PhonemgrListener *l);
 void phonemgr_listener_answer_call	(PhonemgrListener *l);
 void phonemgr_listener_set_time		(PhonemgrListener *l,
